#!/usr/bin/python

import os
import sys
import subprocess
import stat
import atexit
import shutil
import tempfile
from distutils import spawn
from itertools import chain

# the juju-deployer/ directory.
deployer_dir = os.path.abspath(os.path.dirname(sys.argv[0]))


def check_environment():
    if not os.environ.get('OS_USERNAME'):
        print >>sys.stderr, "Please source a novarc file."
        sys.exit(1)


def check_dependencies():
    '''Exit the application if we are missing any dependency programs'''

    needed_commands = ['cheetah']
    for cmd in needed_commands:
        if not spawn.find_executable(cmd):
            err = "%s is missing. Please install it before continuing." % cmd
            print >>sys.stderr, err
            sys.exit(1)


def populate_templates(deployer_dir, working_dir):
    # The command to populate the template.
    cmd_base = ['cheetah', 'fill', '--env', '-p']

    for tmpl in os.listdir(deployer_dir):
        path = os.path.join(deployer_dir, tmpl)
        if os.path.isdir(path):
            src = os.path.join(deployer_dir, tmpl)
            dst = os.path.join(working_dir, tmpl)
            populate_templates(src, dst)
            continue
        if not tmpl.endswith('.yaml.tmpl'):
            continue
        # Strip the .tmpl off .yaml.tmpl and write a novarc-populated version
        # of the yaml to our working directory.
        with open(os.path.join(working_dir, tmpl[:-5]), 'w') as out:
            args = [os.path.join(deployer_dir, tmpl)]
            subprocess.check_call(cmd_base + args, stdout=out)


def generate_argument_list(d, specific=None):
    for f in os.listdir(d):
        if not f.endswith('.yaml'):
            continue
        # If we're looking for a specific set of components to deploy and this
        # isn't one of them, continue.
        if specific and f[:-5] not in specific:
            continue
        yield ('-c', os.path.join(d, f))


def append_python_path(path):
    path = os.path.abspath(path)
    if os.environ.get('PYTHONPATH'):
        os.environ['PYTHONPATH'] += ':%s' % path
    else:
        os.environ['PYTHONPATH'] = '%s' % path


def install(branch):
    """Put a copy of a bzr branch in a temporary directory."""

    temp_dir = tempfile.mkdtemp()
    try:
        subprocess.check_call(['bzr', 'co', '--lightweight', branch, temp_dir])
        append_python_path(temp_dir)
    except:
        shutil.rmtree(temp_dir)
        raise

    return temp_dir


def install_deployer():
    """Install juju-deployer from bzr. We need juju-deployer from tip to have
    JUJU_REPOSITORY work properly (r87)."""

    deployer = install('lp:juju-deployer')
    os.environ['PATH'] = '%s:%s' % (deployer, os.environ['PATH'])
    deployer_bin = os.path.join(deployer, 'juju-deployer')
    with open(deployer_bin, 'w') as fp:
        tmpl = '#!/bin/sh\nPYTHONPATH=%s\nexec python %s/deployer/cli.py $*'
        fp.write(tmpl % (os.environ['PYTHONPATH'], deployer))
    st = os.stat(deployer_bin)
    mode = st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
    os.chmod(deployer_bin, mode)
    return deployer


def remove_branch(branch_dir):
    """Clean up the bzr branch if we didn't already do so as a result of
    bzr failing."""

    if os.path.exists(branch_dir):
        shutil.rmtree(branch_dir)


def build_charms(charmsdir=None):
    # juju needs to know about our local charms.
    # and some of our charms require us to call "make" on them
    if not charmsdir:
        charmsdir = os.path.abspath(
            os.path.join(os.path.dirname(__file__), '../charms'))
    os.environ['JUJU_REPOSITORY'] = charmsdir

    def _cleanup(local_charms):
        # delete any charms pulled down by juju-deployer into our local tree
        if not os.path.exists(charmsdir):
            return  # handle special case for unit test harness
        for release in local_charms:
            charms = os.listdir(os.path.join(charmsdir, release))
            remove = set(charms) - set(local_charms[release])
            for r in remove:
                shutil.rmtree(os.path.join(charmsdir, release, r))

    local_charms = {}
    clean_func = lambda: _cleanup(local_charms)
    atexit.register(clean_func)

    for release in os.listdir(charmsdir):
        local_charms[release] = []
        for charm in os.listdir(os.path.join(charmsdir, release)):
            local_charms[release].append(charm)
            charmdir = os.path.join(charmsdir, release, charm)
            if os.path.exists(os.path.join(charmdir, 'Makefile')):
                print('Building charm: %s' % charm)
                subprocess.check_call(['make'], cwd=charmdir)
    return clean_func


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    # We only want to deploy some specific deployer configs.
    specific = args

    check_environment()
    check_dependencies()

    temp_dir = install_deployer()
    atexit.register(lambda: remove_branch(temp_dir))

    # A place to put our generated .yaml files.
    temp_dir = tempfile.mkdtemp()
    atexit.register(lambda: shutil.rmtree(temp_dir))
    working_dir = os.path.join(temp_dir, 'deployer')
    shutil.copytree(deployer_dir, working_dir)

    # Create .yaml files from .yaml.tmpl files.
    populate_templates(deployer_dir, working_dir)

    # Generate an argument list of -c FILE for each .yaml file in our working
    # directory.
    i = generate_argument_list(working_dir, specific)
    generated = list(chain.from_iterable(i))

    build_charms()

    args = ['juju-deployer', '-v'] + generated + ['ci-airline-staging']
    print 'Running %s' % ' '.join(args)
    subprocess.check_call(args, cwd=os.path.join(deployer_dir, '../charms'))

if __name__ == '__main__':
    main()
